@@ -3,10 +3,22 @@
 import sys
 import argparse
 import json
+
 from collections import defaultdict
+from statistics import median
 
-from typing import (Set, Dict, List, Union)
+from typing import (Set, Dict, List, Sequence, Union)
 
+import numpy as np
+import pandas as pd
+import xgboost as xgb
+
+from predectorutils.data import (
+    get_interesting_dbcan_ids,
+    get_interesting_pfam_ids,
+    get_ltr_model,
+)
+from predectorutils.gff import GFFRecord
 from predectorutils.analyses import (
     Analyses,
     Analysis,
@@ -32,7 +44,9 @@ from predectorutils.analyses import (
 
 COLUMNS = [
     "name",
-    "score",
+    "effector_score",
+    "manual_effector_score",
+    "manual_secretion_score",
     "phibase_effector",
     "phibase_virulence",
     "phibase_lethal",
@@ -44,8 +58,15 @@ COLUMNS = [
     "pfam_matches",
     "dbcan_match",
     "dbcan_matches",
+    "effectorp1",
+    "effectorp2",
+    "is_secreted",
+    "any_signal_peptide",
+    "apoplastp",
+    "single_transmembrane",
+    "multiple_transmembrane",
     "molecular_weight",
-    "residues",
+    "residue_number",
     "charge",
     "isoelectric_point",
     "aa_c_number",
@@ -57,14 +78,10 @@ COLUMNS = [
     "aa_charged_number",
     "aa_basic_number",
     "aa_acidic_number",
-    "effectorp1",
-    "effectorp2",
-    "apoplastp",
+    "fykin_gap",
     "localizer_nuclear",
     "localizer_chloro",
     "localizer_mito",
-    "is_secreted",
-    "any_signal_peptide",
     "signalp3_nn",
     "signalp3_hmm",
     "signalp4",
@@ -73,9 +90,12 @@ COLUMNS = [
     "phobius_sp",
     "phobius_tmcount",
     "tmhmm_tmcount",
-    "tmhmm_first60",
+    "tmhmm_first_60",
+    "tmhmm_exp_aa",
+    "tmhmm_first_tm_sp_coverage",
     "targetp_secreted",
-    "targetp_mitochondrial",
+    "targetp_secreted_prob",
+    "targetp_mitochondrial_prob",
     "deeploc_membrane",
     "deeploc_nucleus",
     "deeploc_cytoplasm",
@@ -90,81 +110,9 @@ COLUMNS = [
     "signalp3_nn_d",
     "signalp3_hmm_s",
     "signalp4_d",
-    "signalp4_dmax_cut",
     "signalp5_prob",
 ]
 
-DBCAN_DEFAULT = {
-    "CBM50",
-    "AA9",
-    "GH10",
-    "GH11"
-}
-
-
-PFAM_DEFAULT = {
-    "PF00734", "PF03443", "PF01083", "PF03211", "PF00331",
-    "PF00457", "PF00160", "PF04126", "PF10231", "PF18050",
-    "PF00188", "PF00293", "PF00964", "PF03377", "PF05730",
-    "PF07249", "PF16541", "PF03330", "PF11529", "PF18661",
-    "PF18247", "PF18241", "PF14521", "PF11327", "PF01764",
-    "PF14856", "PF08995", "PF05630", "PF09461", "PF11584",
-    "PF18224", "PF00187", "PF01476", "PF01607", "PF02128",
-    "PF07504", "PF02221", "PF01363", "PF06602", "PF16810",
-    "PF18488", "PF18634", "PF00161", "PF00545", "PF00652",
-    "PF11663", "PF14200", "PF16536", "PF16542", "PF00014",
-    "PF00030", "PF00059", "PF00068", "PF00087", "PF00200",
-    "PF00451", "PF00537", "PF00555", "PF00706", "PF00819",
-    "PF01117", "PF01123", "PF01324", "PF01335", "PF01338",
-    "PF01375", "PF01376", "PF01421", "PF01742", "PF01821",
-    "PF02048", "PF02258", "PF02452", "PF02691", "PF02763",
-    "PF02764", "PF02794", "PF02819", "PF02876", "PF02917",
-    "PF02918", "PF02950", "PF03077", "PF03318", "PF03440",
-    "PF03495", "PF03497", "PF03498", "PF03505", "PF03507",
-    "PF03768", "PF03769", "PF03784", "PF03944", "PF03945",
-    "PF04221", "PF04365", "PF04829", "PF05015", "PF05016",
-    "PF05294", "PF05353", "PF05374", "PF05394", "PF05453",
-    "PF05627", "PF05638", "PF05707", "PF05791", "PF05819",
-    "PF05932", "PF06254", "PF06255", "PF06261", "PF06286",
-    "PF06296", "PF06340", "PF06357", "PF06414", "PF06416",
-    "PF06451", "PF06453", "PF06457", "PF06755", "PF06769",
-    "PF07254", "PF07365", "PF07442", "PF07473", "PF07591",
-    "PF07737", "PF07740", "PF07771", "PF07822", "PF07829",
-    "PF07906", "PF07927", "PF07936", "PF07945", "PF07951",
-    "PF07952", "PF07953", "PF07968", "PF08024", "PF08025",
-    "PF08086", "PF08087", "PF08088", "PF08089", "PF08090",
-    "PF08091", "PF08092", "PF08093", "PF08094", "PF08095",
-    "PF08096", "PF08097", "PF08098", "PF08099", "PF08104",
-    "PF08115", "PF08116", "PF08119", "PF08120", "PF08121",
-    "PF08131", "PF08178", "PF08249", "PF08251", "PF08396",
-    "PF08493", "PF08562", "PF08843", "PF08845", "PF08888",
-    "PF09009", "PF09075", "PF09101", "PF09102", "PF09119",
-    "PF09131", "PF09156", "PF09207", "PF09275", "PF09276",
-    "PF09407", "PF09857", "PF09907", "PF10279", "PF10530",
-    "PF10550", "PF11047", "PF11410", "PF11592", "PF12207",
-    "PF12255", "PF12256", "PF12563", "PF12703", "PF12918",
-    "PF13304", "PF13939", "PF13940", "PF13955", "PF13956",
-    "PF13957", "PF13958", "PF13979", "PF13981", "PF14021",
-    "PF14113", "PF14171", "PF14449", "PF14866", "PF15500",
-    "PF15520", "PF15521", "PF15522", "PF15523", "PF15524",
-    "PF15526", "PF15527", "PF15528", "PF15529", "PF15530",
-    "PF15531", "PF15532", "PF15533", "PF15534", "PF15535",
-    "PF15536", "PF15537", "PF15538", "PF15540", "PF15541",
-    "PF15542", "PF15543", "PF15544", "PF15545", "PF15604",
-    "PF15605", "PF15606", "PF15607", "PF15635", "PF15636",
-    "PF15637", "PF15638", "PF15639", "PF15640", "PF15641",
-    "PF15643", "PF15644", "PF15645", "PF15646", "PF15647",
-    "PF15648", "PF15649", "PF15650", "PF15651", "PF15652",
-    "PF15653", "PF15657", "PF15658", "PF15659", "PF15723",
-    "PF15738", "PF15781", "PF15935", "PF16754", "PF16847",
-    "PF16873", "PF16981", "PF17056", "PF17454", "PF17475",
-    "PF17476", "PF17486", "PF17491", "PF17492", "PF17499",
-    "PF17521", "PF17556", "PF17557", "PF17563", "PF17997",
-    "PF18022", "PF18078", "PF18276", "PF18449", "PF18648",
-    "PF18807", "PF06766", "PF16073", "PF11807", "PF00314",
-    "PF16361"
-}
-
 
 def cli(parser: argparse.ArgumentParser) -> None:
 
@@ -196,145 +144,165 @@ def cli(parser: argparse.ArgumentParser) -> None:
     )
 
     parser.add_argument(
-        "--secreted-score",
+        "--secreted-weight",
         type=float,
         default=3,
-        help="The score to give a protein if it is predicted to be secreted."
+        help=(
+            "The weight to give a protein if it is predicted to be secreted "
+            "by any signal peptide method."
+        )
     )
 
     parser.add_argument(
-        "--sigpep-good-score",
+        "--sigpep-good-weight",
         type=float,
         default=0.5,
         help=(
-            "The score to give a protein if it is predicted to have a signal "
+            "The weight to give a protein if it is predicted to have a signal "
             "peptide by one of the more reliable methods."
         )
     )
 
     parser.add_argument(
-        "--sigpep-ok-score",
+        "--sigpep-ok-weight",
         type=float,
         default=0.25,
         help=(
-            "The score to give a protein if it is predicted to have a signal "
+            "The weight to give a protein if it is predicted to have a signal "
             "peptide by one of the reasonably reliable methods."
         )
     )
 
     parser.add_argument(
-        "--transmembrane-score",
+        "--single-transmembrane-weight",
         type=float,
-        default=-10,
+        default=-2,
         help=(
-            "The score to give a protein if it is predicted to be "
-            "transmembrane. Use negative numbers to penalise."
+            "The weight to give a protein if it is predicted to have "
+            "> 0 TM domains by either method and not both > 1 (mutually "
+            "exclusive with multiple-transmembrane-score). "
+            "This is not applied if TMHMM first 60 is > 10. "
+            "Use negative numbers to penalise."
         )
     )
 
     parser.add_argument(
-        "--deeploc-extracellular-score",
+        "--multiple-transmembrane-weight",
+        type=float,
+        default=-6,
+        help=(
+            "The weight to give a protein if it is predicted to have"
+            "transmembrane have > 1 TM domains by both TMHMM and Phobius."
+            "Use negative numbers to penalise."
+        )
+    )
+
+    parser.add_argument(
+        "--deeploc-extracellular-weight",
         type=float,
         default=1,
         help=(
-            "The score to give a protein if it is predicted to be "
+            "The weight to give a protein if it is predicted to be "
             "extracellular by deeploc."
         )
     )
 
     parser.add_argument(
-        "--deeploc-intracellular-score",
+        "--deeploc-intracellular-weight",
         type=float,
-        default=-2,
+        default=-0.5,
         help=(
-            "The score to give a protein if it is predicted to be "
+            "The weigt to give a protein if it is predicted to be "
             "intracellular by deeploc. Use negative numbers to penalise."
         )
     )
 
     parser.add_argument(
-        "--targetp-secreted-score",
+        "--deeploc-membrane-weight",
         type=float,
-        default=1,
+        default=-1,
         help=(
-            "The score to give a protein if it is predicted to be "
-            "secreted by targetp."
+            "The weight to give a protein if it is predicted to be "
+            "membrane associated by deeploc. Use negative numbers to penalise."
         )
     )
 
     parser.add_argument(
-        "--targetp-mitochondrial-score",
+        "--targetp-mitochondrial-weight",
         type=float,
-        default=-2,
+        default=-0.5,
         help=(
-            "The score to give a protein if it is predicted to be "
+            "The weight to give a protein if it is predicted to be "
             "mitochondrial by targetp. Use negative numbers to penalise."
         )
     )
 
     parser.add_argument(
-        "--effectorp1-score",
+        "--effectorp1-weight",
         type=float,
         default=3,
         help=(
-            "The score to give a protein if it is predicted to be "
+            "The weight to give a protein if it is predicted to be "
             "an effector by effectorp1."
         )
     )
 
     parser.add_argument(
-        "--effectorp2-score",
+        "--effectorp2-weight",
         type=float,
         default=3,
         help=(
-            "The score to give a protein if it is predicted to be "
+            "The weight to give a protein if it is predicted to be "
             "an effector by effectorp2."
         )
     )
 
     parser.add_argument(
-        "--effector-homology-score",
+        "--effector-homology-weight",
         type=float,
         default=5,
         help=(
-            "The score to give a protein if it is similar to a known "
+            "The weight to give a protein if it is similar to a known "
             "effector or effector domain."
         )
     )
 
     parser.add_argument(
-        "--virulence-homology-score",
+        "--virulence-homology-weight",
         type=float,
         default=1,
         help=(
-            "The score to give a protein if it is similar to a known "
+            "The weight to give a protein if it is similar to a known "
             "protein that may be involved in virulence."
         )
     )
 
     parser.add_argument(
-        "--lethal-homology-score",
+        "--lethal-homology-weight",
         type=float,
         default=-5,
         help=(
-            "The score to give a protein if it is similar to a known "
+            "The weight to give a protein if it is similar to a known "
             "protein in phibase which caused a lethal phenotype."
         )
     )
 
+    parser.add_argument(
+        "--tmhmm-first-60-threshold",
+        type=float,
+        default=10,
+        help=(
+            "The minimum number of AAs predicted to be transmembrane in the "
+            "first 60 AAs to consider a protein with a single TM domain "
+            "a false positive (caused by hydrophobic region in sp)."
+        )
+    )
+
     return
 
 
-def score_it(
+def effector_score_it(
     record: Dict[str, Union[None, int, float, str]],
-    secreted: float = 3,
-    less_trustworthy_signal_prediction: float = 0.25,
-    trustworthy_signal_prediction: float = 0.5,
-    transmembrane: float = -10,
-    deeploc_extracellular: float = 1,
-    deeploc_intracellular: float = -2,
-    targetp_secreted: float = 1,
-    targetp_mitochondrial: float = -2,
     effectorp1: float = 3,
     effectorp2: float = 3,
     effector: float = 5,
@@ -342,61 +310,26 @@ def score_it(
     lethal: float = -5,
 ) -> float:
     """ """
-
     score: float = 0
 
-    is_secreted = record.get("is_secreted", 0)
-    assert isinstance(is_secreted, int)
-    score += int(is_secreted) * secreted
+    secretion_score = record["manual_secretion_score"]
+    assert isinstance(secretion_score, float)
+    score += secretion_score
 
-    for k in ["signalp3_hmm", "signalp3_nn", "phobius_sp"]:
-        v = record.get(k, 0)
-        assert isinstance(v, int)
-        score += v * less_trustworthy_signal_prediction
-
-    for k in ["signalp4", "signalp5", "deepsig"]:
-        v = record.get(k, 0)
-        assert isinstance(v, int)
-        score += v * trustworthy_signal_prediction
-
-    is_transmembrane = record.get("is_transmembrane", 0)
-    assert isinstance(is_transmembrane, int)
-    score += is_transmembrane * transmembrane
-
-    deeploc_extracellular_prob = record.get("deeploc_extracellular", 0.0)
-    assert isinstance(deeploc_extracellular_prob, float)
-    score += deeploc_extracellular_prob * deeploc_extracellular  # noqa
-    for k in [
-        'deeploc_membrane', 'deeploc_nucleus', 'deeploc_cytoplasm',
-        'deeploc_mitochondrion', 'deeploc_cell_membrane', 'deeploc_plastid',
-        'deeploc_lysosome', 'deeploc_peroxisome'
-    ]:
-        v = record.get(k, 0.0)
-        assert isinstance(v, float)
-        score += v * deeploc_intracellular
-
-    targetp_secreted_prob = record.get("targetp_secreted", 0.0)
-    targetp_mitochondrial_prob = record.get("targetp_mitochondrial", 0.0)
-
-    assert isinstance(targetp_secreted_prob, float)
-    assert isinstance(targetp_mitochondrial_prob, float)
-    score += targetp_secreted_prob * targetp_secreted
-    score += targetp_mitochondrial_prob * targetp_mitochondrial
-
-    effectorp1_prob = record.get("effectorp1", 0.0)
-    effectorp2_prob = record.get("effectorp2", 0.0)
+    effectorp1_prob = record["effectorp1"]
+    effectorp2_prob = record["effectorp2"]
 
     assert isinstance(effectorp1_prob, float)
     assert isinstance(effectorp2_prob, float)
     score += 2 * (effectorp1_prob - 0.5) * effectorp1
     score += 2 * (effectorp2_prob - 0.5) * effectorp2
 
-    assert isinstance(record["phibase_effector"], int)
+    assert isinstance(record["phibase_effector_match"], int)
     assert isinstance(record["effector_match"], int)
     assert isinstance(record["dbcan_match"], int)
     assert isinstance(record["pfam_match"], int)
     has_effector_match = sum([
-        record["phibase_effector"],
+        record["phibase_effector_match"],
         record["effector_match"],
         record["dbcan_match"],
         record["pfam_match"]
@@ -404,13 +337,70 @@ def score_it(
 
     score += int(has_effector_match) * effector
 
-    assert isinstance(record["phibase_virulence"], int)
+    assert isinstance(record["phibase_virulence_match"], int)
     if not has_effector_match:
-        score += record["phibase_virulence"] * virulence
+        score += record["phibase_virulence_match"] * virulence
 
-    assert isinstance(record["phibase_lethal"], int)
-    score += record["phibase_lethal"] * lethal
+    assert isinstance(record["phibase_lethal_match"], int)
+    score += record["phibase_lethal_match"] * lethal
+    return score
 
+
+def secretion_score_it(
+    record: Dict[str, Union[None, int, float, str]],
+    secreted: float = 3,
+    less_trustworthy_signal_prediction: float = 0.25,
+    trustworthy_signal_prediction: float = 0.5,
+    single_transmembrane: float = -1,
+    multiple_transmembrane: float = -10,
+    deeploc_extracellular: float = 1,
+    deeploc_intracellular: float = -2,
+    deeploc_membrane: float = -2,
+    targetp_mitochondrial: float = -2,
+) -> float:
+    score: float = 0
+
+    assert isinstance(record["is_secreted"], int)
+    score += int(record["is_secreted"]) * secreted
+
+    for k in ["signalp3_hmm", "signalp3_nn", "phobius_sp", "deepsig"]:
+        v = record.get(k, 0)
+        assert isinstance(v, int)
+        score += v * less_trustworthy_signal_prediction
+
+    for k in ["signalp4", "signalp5", "targetp_secreted"]:
+        v = record.get(k, 0)
+        assert isinstance(v, int)
+        score += v * trustworthy_signal_prediction
+
+    assert isinstance(record["multiple_transmembrane"], int)
+    score += record["multiple_transmembrane"] * multiple_transmembrane
+
+    assert isinstance(record["single_transmembrane"], int)
+    score += record["single_transmembrane"] * single_transmembrane
+
+    assert isinstance(record["deeploc_extracellular"], float)
+    score += record["deeploc_extracellular"] * deeploc_extracellular  # noqa
+    for k in [
+        'deeploc_nucleus',
+        'deeploc_cytoplasm',
+        'deeploc_mitochondrion',
+        'deeploc_cell_membrane',
+        'deeploc_endoplasmic_reticulum',
+        'deeploc_plastid',
+        'deeploc_golgi',
+        'deeploc_lysosome',
+        'deeploc_peroxisome'
+    ]:
+        v = record.get(k, 0.0)
+        assert isinstance(v, float)
+        score += v * deeploc_intracellular
+
+    assert isinstance(record["deeploc_membrane"], float)
+    score += record["deeploc_membrane"] * deeploc_membrane
+
+    assert isinstance(record["targetp_mitochondrial_prob"], float)
+    score += record["targetp_mitochondrial_prob"] * targetp_mitochondrial
     return score
 
 
@@ -440,28 +430,83 @@ def decide_any_signal(
         for k
         in [
             'signalp3_nn', 'signalp3_hmm', 'signalp4',
-            'signalp5', 'deepsig', 'phobius_sp'
+            'signalp5', 'deepsig', 'phobius_sp', 'targetp_secreted'
         ]
     ]))
     return
 
 
+def gff_intersection(left: GFFRecord, right: GFFRecord) -> int:
+    lstart = min([left.start, left.end])
+    lend = max([left.start, left.end])
+
+    rstart = min([right.start, right.end])
+    rend = max([right.start, right.end])
+
+    start = max([lstart, rstart])
+    end = min([lend, rend])
+
+    # This will be < 0 if they don't overlap
+    if start < end:
+        return end - start
+    else:
+        return 0
+
+
+def gff_coverage(left: GFFRecord, right: GFFRecord) -> float:
+    noverlap = gff_intersection(left, right)
+    return noverlap / (right.end - right.start)
+
+
+def get_tm_sp_coverage(
+    sp_gff: Sequence[GFFRecord],
+    tm_gff: Sequence[GFFRecord],
+) -> float:
+    if len(sp_gff) == 0:
+        return 0.0
+
+    if len(tm_gff) == 0:
+        return 0.0
+
+    tm = sorted(tm_gff, key=lambda x: x.start)[0]
+    covs = [gff_coverage(sp, tm) for sp in sp_gff]
+    return median(covs)
+
+
 def decide_is_transmembrane(
-    record: Dict[str, Union[None, int, float, str]]
+    record: Dict[str, Union[None, int, float, str]],
+    sp_gff: Sequence[GFFRecord],
+    tm_gff: Sequence[GFFRecord],
+    tmhmm_first_60_threshold: float = 10,
 ) -> None:
 
     assert isinstance(record["tmhmm_tmcount"], int)
     assert isinstance(record["phobius_tmcount"], int)
-    assert isinstance(record["tmhmm_first60"], float)
-    assert isinstance(record["any_signal_peptide"], int)
+    assert isinstance(record["tmhmm_first_60"], float)
 
-    record["is_transmembrane"] = int(
+    record["tmhmm_first_tm_sp_coverage"] = get_tm_sp_coverage(sp_gff, tm_gff)
+
+    record["multiple_transmembrane"] = int(
         (record["tmhmm_tmcount"] > 1) or
-        (record["phobius_tmcount"] > 0) or
-        ((record["tmhmm_first60"] > 10) and
-         (record["tmhmm_tmcount"] == 1) and
-         bool(record["any_signal_peptide"]))
+        (record["phobius_tmcount"] > 1)
     )
+
+    record["single_transmembrane"] = int(
+        not bool(record["multiple_transmembrane"])
+        and (
+            record["phobius_tmcount"] == 1
+            or (
+                record["tmhmm_tmcount"] == 1
+                and not bool(record["any_signal_peptide"])
+            )
+            or (
+                record["tmhmm_tmcount"] == 1
+                and bool(record["any_signal_peptide"])
+                and (record["tmhmm_first_60"] < tmhmm_first_60_threshold)
+            )
+        )
+    )
+
     return
 
 
@@ -470,7 +515,7 @@ def decide_is_secreted(
 ) -> None:
     record["is_secreted"] = int(
         bool(record["any_signal_peptide"]) and not
-        bool(record["is_transmembrane"])
+        bool(record["multiple_transmembrane"])
     )
     return
 
@@ -498,7 +543,7 @@ def get_pepstats_cols(
     record: Dict[str, Union[None, int, float, str]]
 ) -> None:
     record["molecular_weight"] = an.molecular_weight
-    record["residues"] = an.residues
+    record["residue_number"] = an.residues
     record["charge"] = an.charge
     record["isoelectric_point"] = an.isoelectric_point
     record["aa_c_number"] = an.residue_c_number
@@ -510,6 +555,22 @@ def get_pepstats_cols(
     record["aa_charged_number"] = an.property_charged_number
     record["aa_basic_number"] = an.property_basic_number
     record["aa_acidic_number"] = an.property_acidic_number
+
+    fykin = (
+        an.residue_f_number +
+        an.residue_k_number +
+        an.residue_y_number +
+        an.residue_i_number +
+        an.residue_n_number
+    )
+
+    gap = (
+        an.residue_g_number +
+        an.residue_a_number +
+        an.residue_p_number
+    )
+
+    record["fykin_gap"] = (float(fykin) + 1) / (float(gap) + 1)
     return
 
 
@@ -518,7 +579,7 @@ def get_phibase_cols(
     phenotypes: Set[str],
     record: Dict[str, Union[None, int, float, str]]
 ) -> None:
-    record["phibase_effector"] = int(len(
+    record["phibase_effector_match"] = int(len(
         phenotypes.intersection([
             "loss_of_pathogenicity",
             "increased_virulence_(hypervirulence)",
@@ -526,8 +587,8 @@ def get_phibase_cols(
         ])
     ) > 0)
 
-    record["phibase_virulence"] = int("reduced_virulence" in phenotypes)
-    record["phibase_lethal"] = int("lethal" in phenotypes)
+    record["phibase_virulence_match"] = int("reduced_virulence" in phenotypes)
+    record["phibase_lethal_match"] = int("lethal" in phenotypes)
 
     if len(phenotypes) > 0:
         record["phibase_phenotypes"] = ",".join(phenotypes)
@@ -546,11 +607,13 @@ def get_sper_prob_col(
     else:
         return 1 - an.prob
 
+
 def construct_row(  # noqa
     name,
     analyses: List[Analysis],
     pfam_domains: Set[str],
-    dbcan_domains: Set[str]
+    dbcan_domains: Set[str],
+    tmhmm_first_60_threshold: float,
 ) -> Dict[str, Union[None, int, float, str]]:
 
     phibase_matches: Set[str] = set()
@@ -562,50 +625,80 @@ def construct_row(  # noqa
     record: Dict[str, Union[None, int, float, str]] = {"name": name}
     record["effector_match"] = 0
 
+    tm_gff: List[GFFRecord] = []
+    sp_gff: List[GFFRecord] = []
+
     for an in analyses:
         if isinstance(an, ApoplastP):
             record["apoplastp"] = get_sper_prob_col(an, ["Apoplastic"])
+
         elif isinstance(an, DeepSig):
             record["deepsig"] = int(an.prediction == "SignalPeptide")
+            sp_gff.extend(an.as_gff())
+
         elif isinstance(an, EffectorP1):
             record["effectorp1"] = get_sper_prob_col(an, ["Effector"])
+
         elif isinstance(an, EffectorP2):
             record["effectorp2"] = get_sper_prob_col(
                 an,
                 ["Effector", "Unlikely effector"]
             )
+
         elif isinstance(an, Phobius):
             record["phobius_sp"] = int(an.sp)
             record["phobius_tmcount"] = an.tm
+            sp_gff.extend(
+                r for r in an.as_gff()
+                if r.type == "signal_peptide"
+            )
+
         elif isinstance(an, SignalP3HMM):
             record["signalp3_hmm"] = int(an.is_secreted)
             record["signalp3_hmm_s"] = an.sprob
+            sp_gff.extend(an.as_gff())
+
         elif isinstance(an, SignalP3NN):
             record["signalp3_nn"] = int(an.d_decision)
             record["signalp3_nn_d"] = an.d
+            sp_gff.extend(an.as_gff())
+
         elif isinstance(an, SignalP4):
             record["signalp4"] = int(an.decision)
             record["signalp4_d"] = an.d
             record["signalp4_dmax_cut"] = an.dmax_cut
+            sp_gff.extend(an.as_gff())
 
         elif isinstance(an, SignalP5):
             record["signalp5"] = int(an.prediction == "SP(Sec/SPI)")
-            record["signalp5_prob"] = an.prob_signal
+            # For some proteins, this outputs a very high number
+            # so we constrain it here.
+            record["signalp5_prob"] = min([an.prob_signal, 1.0])
+            sp_gff.extend(an.as_gff())
+
         elif isinstance(an, TargetPNonPlant):
-            record["targetp_secreted"] = an.sp
-            record["targetp_mitochondrial"] = an.mtp
+            record["targetp_secreted"] = int(an.prediction == "SP")
+            record["targetp_secreted_prob"] = an.sp
+            record["targetp_mitochondrial_prob"] = an.mtp
+
         elif isinstance(an, TMHMM):
             record["tmhmm_tmcount"] = an.pred_hel
-            record["tmhmm_first60"] = an.first_60
+            record["tmhmm_first_60"] = an.first_60
+            record["tmhmm_exp_aa"] = an.exp_aa
+            tm_gff.extend(an.as_gff())
+
         elif isinstance(an, LOCALIZER):
             record["localizer_nuclear"] = int(an.nucleus_decision)
             record["localizer_chloro"] = int(an.chloroplast_decision)
             record["localizer_mito"] = int(an.mitochondria_decision)
+
         elif isinstance(an, DeepLoc):
             get_deeploc_cols(an, record)
+
         elif isinstance(an, DBCAN):
             if an.decide_significant():
                 dbcan_matches.add(an.hmm)
+
         elif isinstance(an, PfamScan):
             hmm = an.hmm.split('.', maxsplit=1)[0]
             if hmm in pfam_domains:
@@ -625,7 +718,7 @@ def construct_row(  # noqa
                 effector_matches.add(an.target)
 
     decide_any_signal(record)
-    decide_is_transmembrane(record)
+    decide_is_transmembrane(record, sp_gff, tm_gff, tmhmm_first_60_threshold)
     decide_is_secreted(record)
 
     get_phibase_cols(phibase_matches, phibase_phenotypes, record)
@@ -650,6 +743,52 @@ def construct_row(  # noqa
     return record
 
 
+def run_ltr(df: pd.DataFrame) -> np.array:
+    df = df.copy()
+
+    df["aa_c_prop"] = df["aa_c_number"] / df["residue_number"]
+    df["aa_tiny_prop"] = df["aa_tiny_number"] / df["residue_number"]
+    df["aa_small_prop"] = df["aa_small_number"] / df["residue_number"]
+    df["aa_aliphatic_prop"] = df["aa_aliphatic_number"] / df["residue_number"]
+    df["aa_aromatic_prop"] = df["aa_aromatic_number"] / df["residue_number"]
+    df["aa_nonpolar_prop"] = df["aa_nonpolar_number"] / df["residue_number"]
+    df["aa_charged_prop"] = df["aa_charged_number"] / df["residue_number"]
+    df["aa_basic_prop"] = df["aa_basic_number"] / df["residue_number"]
+    df["aa_acidic_prop"] = df["aa_acidic_number"] / df["residue_number"]
+
+    df_features = df[[
+        'molecular_weight',
+        'aa_c_prop',
+        'aa_tiny_prop',
+        'aa_small_prop',
+        'aa_nonpolar_prop',
+        'aa_basic_prop',
+        'effectorp1',
+        'effectorp2',
+        'apoplastp',
+        'phobius_tmcount',
+        'tmhmm_tmcount',
+        'tmhmm_first_60',
+        'deeploc_membrane',
+        'deeploc_extracellular',
+        'deepsig',
+        'phobius_sp',
+        'signalp3_nn_d',
+        'signalp3_hmm_s',
+        'signalp4_d',
+        'signalp5_prob',
+        'targetp_secreted_prob',
+    ]]
+
+    dmat = xgb.DMatrix(df_features)
+    model = get_ltr_model()
+    return model.predict(dmat)
+
+
+def record_to_series(record: Dict[str, Union[None, int, float, str]]) -> str:
+    return pd.Series([record.get(c, None) for c in COLUMNS], index=COLUMNS)
+
+
 def write_line(record: Dict[str, Union[None, int, float, str]]) -> str:
     line = "\t".join(str(record.get(c, ".")) for c in COLUMNS)
     return line
@@ -659,10 +798,14 @@ def runner(args: argparse.Namespace) -> None:
     records = defaultdict(list)
 
     if args.dbcan is not None:
-        dbcan = {l.strip() for l in args.dbcan.readlines()}
+        dbcan: Set[str] = {d.strip() for d in args.dbcan.readlines()}
+    else:
+        dbcan = set(get_interesting_dbcan_ids())
 
     if args.pfam is not None:
-        pfam = {l.strip() for l in args.pfam.readlines()}
+        pfam: Set[str] = {d.strip() for d in args.pfam.readlines()}
+    else:
+        pfam = set(get_interesting_pfam_ids())
 
     for line in args.infile:
         sline = line.strip()
@@ -674,26 +817,42 @@ def runner(args: argparse.Namespace) -> None:
         records[dline["protein_name"]].append(record)
 
     print("\t".join(COLUMNS), file=args.outfile)
+    out_records = []
 
     for name, protein_records in records.items():
-        record = construct_row(name, protein_records, pfam, dbcan)
-        record["score"] = score_it(
-            record,
-            args.secreted_score,
-            args.sigpep_ok_score,
-            args.sigpep_good_score,
-            args.transmembrane_score,
-            args.deeploc_extracellular_score,
-            args.deeploc_intracellular_score,
-            args.targetp_secreted_score,
-            args.targetp_mitochondrial_score,
-            args.effectorp1_score,
-            args.effectorp2_score,
-            args.effector_homology_score,
-            args.virulence_homology_score,
-            args.lethal_homology_score,
+        record = construct_row(
+            name,
+            protein_records,
+            pfam,
+            dbcan,
+            args.tmhmm_first_60_threshold
         )
 
-        line = write_line(record)
-        print(line, file=args.outfile)
+        record["manual_secretion_score"] = secretion_score_it(
+            record,
+            args.secreted_weight,
+            args.sigpep_ok_weight,
+            args.sigpep_good_weight,
+            args.single_transmembrane_weight,
+            args.multiple_transmembrane_weight,
+            args.deeploc_extracellular_weight,
+            args.deeploc_intracellular_weight,
+            args.deeploc_membrane_weight,
+            args.targetp_mitochondrial_weight,
+        )
+        record["manual_effector_score"] = effector_score_it(
+            record,
+            args.effectorp1_weight,
+            args.effectorp2_weight,
+            args.effector_homology_weight,
+            args.virulence_homology_weight,
+            args.lethal_homology_weight,
+        )
+        out_records.append(record)
+
+    df = pd.DataFrame([record_to_series(r) for r in out_records])
+    df["effector_score"] = run_ltr(df)
+    df.sort_values("effector_score", ascending=False, inplace=True)
+
+    df.to_csv(args.outfile, sep="\t", index=False, na_rep=".")
     return
